using System;

namespace com.ibm.gre
{
	/// <summary>
	/// Enumerated constants representing
	/// Records Manager API constants
	/// </summary>
	public class IRMConstants
	{
		public IRMConstants(){}

		//IBM DB2 Records Manager Constants
		
		/// <summary>
		/// Constant values specifying the File Plan Component Type
		/// Copied from the IBM DB2 Records Manager API
		/// com.ibm.gre.engine.util.FilePlanComponentTypes class  
		/// </summary>
		public enum FilePlanComponentTypes 
		{
			COMPONENT = 1,
			RECORD = 2,
			PARTITION = 3
		}
		
		/// <summary>
		/// Constant values specifying the type of Fields Collection
		/// Copied from the IBM DB2 Records Manager API
		/// com.ibm.gre.engine.util.TemplateOptions class  
		/// </summary>
		public enum TemplateOptions 
		{
			/// <summary>
			/// When no bit mask values are set the TemplateController will return a standard Fields collection.
			/// </summary>
			NONE = 0,
			/// <summary>
			/// When this bit is set the TemplateController will return a Fields collection with all List values expanded.
			/// </summary>
			EXPAND_LISTS = 1,
			/// <summary>
			/// When this bit is set the TemplateController will return the Lists of Profiles and Defaults that apply to this component.
			/// </summary>
			PROFILE_FIELDS = 2,	
			/// <summary>
			/// When this bit is set the TemplateController will expand lists and profile fields.
			/// </summary>
			EXPAND_LISTS_AND_PROFILE_FIELDS = 3,		
			/// <summary>
			/// When this bit is set the TemplateController will expand lists and profile fields but will ignore field level permissions.
			/// </summary>
			EXPAND_LISTS_AND_PROFILE_FIELDS_NO_FLS = 4,
		}

		/// <summary>
		/// Constant values specifying the type of result to return for the query
		/// Copied from the IBM DB2 Records Manager API
		/// com.ibm.gre.engine.util.QueryResultOptions class  
		/// </summary>
		public enum QueryResultOptions 
		{
			NORMAL = 0,
			COUNT_ONLY = 1,
			INCLUDE_COLUMNS = 2
		}

		/// <summary>
		/// Constant values specifying the type of log file to be written
		/// Copied from the IBM DB2 Records Manager API
		/// com.ibm.gre.engine.types.LogTypes class  
		/// </summary>
		public enum LogTypes 
		{
			/// <summary>
			/// TRANSITION_LOG this value specifies that
			/// result values will be written to a Life Cycle Processing log file.
			/// This file is an XML document with a "LifeCycleProcessingLog" root element
			/// containing information for each component that is transitioned.
			/// Use this value when getting the results for a batch process that was
			/// started using the
			/// com.ibm.gre.engine.util.AsynchronousTaskType AsynchronousTaskType.TRANSITION_FILE_PLAN_COMPONENTS value.
			/// </summary> 
			TRANSITION_LOG = 1,
			/// <summary>
			/// ERROR_LOG this value specifies that
			/// result values will be written to a Result List log file.
			/// This file is an XML document with a "ResultList" root element
			/// containing error information.
			/// This log value will be used when the Records Manager Returns an Error.
			/// </summary>
			ERROR_LOG = 2,
			/// <summary>
			/// RESULT_LOG this value specifies that
			/// result values will be written to a Result List log file.
			/// This file is an XML document with a "ResultList" root element
			/// containing results form the batch process.
			/// Use this value when getting the results for a batch process that was
			/// started using the
			/// com.ibm.gre.engine.util.AsynchronousTaskType AsynchronousTaskType.GET_ELIGIBLE_FILE_PLAN_COMPONENTS
			/// ,com.ibm.gre.engine.util.AsynchronousTaskType AsynchronousTaskType.PERMISSIONS_SYNCHRONIZATION
			/// , and com.ibm.gre.engine.util.AsynchronousTaskType AsynchronousTaskType.QUERY
			/// values. 
			/// </summary>
			RESULT_LOG = 3,
			/// <summary>
			/// META_DATA_LOG this value specifies that
			/// result values will be written to a File Plan Components log file.
			/// This file is an XML document with a "FilePlanComponents" root element
			/// containing results form the batch process.
			/// This type of file is used if metadata is being transfered during a life cycle transition.
			/// </summary>
			META_DATA_LOG = 4
		}

		/// <summary>
		/// Constant values specifying the results of an Asynchronous task
		/// Copied from the IBM DB2 Records Manager API
		/// com.ibm.gre.engine.util.BatchResultStatusTypes class  
		/// </summary>
		public enum BatchResultStatusTypes 
		{
			/// <summary>
			/// Indicates that the Task Failed.
			/// </summary>
			FAILED = -1,
			/// <summary>
			/// Indicates that the Task has been scheduled but has not yet started.
			/// </summary>
			RECEIVED = -2,
			/// <summary>
			/// Indicates that the Task was Canceled.
			/// </summary>
			CANCELED = -3,
			/// <summary>
			/// Indicates that the Task has started.
			/// </summary>
			STARTED = 0,
			/// <summary>
			/// Indicates that the Task has been scheduled for cancelation.
			/// </summary>
			SCHEDULED_TO_CANCEL = -4,
			/// <summary>
			/// Indicates that the Task completed but there were errors.
			/// </summary>
			COMPLETED_WITH_ERRORS = -5,	
			COMPLETED_PENDING_USER_CONFIRMATION = -6,	
			COMPLETED_WITH_ERRORS_PENDING_USER_CONFIRMATION = -7,	
			COMPLETED = -8,		
			CONVERTING_METADATA_TO_SCHEMA = -9
		}

		/// <summary>
		/// Constant values specifying types of Asynchronous tasks
		/// Copied from the IBM DB2 Records Manager API
		/// com.ibm.gre.engine.util.AsynchronousTaskType class  
		/// </summary>
		public enum AsynchronousTaskType
		{
			/// <summary>
			/// Specifies a Bulk Update Task.
			/// </summary>
			BULK_UPDATE = 1,
			/// <summary>
			/// Specifies a Life Cycle Management Task to Report on Eligable FilePlanComponents.
			/// </summary>
			GET_ELIGIBLE_FILE_PLAN_COMPONENTS = -1,	
			/// <summary>
			/// Specifies a Life Cycle Management Task to Transition a List of FilePlanComponents.
			/// </summary>
			TRANSITION_FILE_PLAN_COMPONENTS = 3,
			/// <summary>
			/// Specifies a Task to Delete a FilePlanComponent and all it's children.
			/// </summary>
			FILE_PLAN_COMPONENT_DELETE = 5,	
			QUERY = 7,	
			CONVERT_TO_XML = 8,
			CONVERT_TO_PDF = 9,	
			PERMISSIONS_SYNCHRONIZATION = 10,
		
			// internal tasks
			FILE_PLAN_COMPONENT_UPDATE = 12,		
			FILE_PLAN_COMPONENT_MOVE = 13,
			PARTITION_DELETE = 14,
			DESTROY_FILE_PLAN_COMPONENTS = 15,
		
			// INTERNAL TASK
			FILE_PLAN_COMPONENT_UPDATE_FOR_SECURITY_CONSISTENCY = 16,
			SEND_EMAIL = 17,
			PURGE_AUDIT_ENTRIES = 18,	
			BULK_UPDATE_ALL = 19,
			QUERY_COUNT = 20,
		
			/// <summary>
			/// Specifies a Bulk Un-suspend Task.
			/// </summary>
			BULK_UNSUSPEND  = 21,
			QUERY_PDF  = 22,
			/// <summary>
			/// Specifies a Bulk Suspend Task.
			/// </summary>
			BULK_SUSPEND  = 23,
		
			// INTERNAL TASK
			CHANGE_PERMISSIONS = 24,	

			// INTERNAL TASK
			LIFE_CYCLE_CODE_PHASE_UPDATE = 25,
			BULK_FILE_PLAN_COMPONENT_MOVE  = 26,	
			CHANGE_PERMISSIONS_FROM_TO = 27		
		}

		/// <summary>
		/// Constant values specifying the type of
		/// Query that will be executed
		/// Copied from the IBM DB2 Records Manager API
		/// com.ibm.gre.engine.types.QueryTypes class  
		/// </summary>
		public enum QueryTypes
		{
			ADVANCED = 1, 
			LIFE_CYCLE = 2
		}

		/// <summary>
		/// Constant values specifying the nameing scheme
		/// Copied from the IBM DB2 Records Manager API
		/// com.ibm.gre.engine.options.DualNamingOptions class  
		/// </summary>
		public enum DualNamingOptions
		{
			BY_NAME = 1,
			BY_CODE = 2
		}

		/// <summary>
		/// Constant values specifying types of Component Attributes
		/// Copied from the IBM DB2 Records Manager API
		/// com.ibm.gre.engine.util.AttributeDataTypes class  
		/// </summary>
		public enum AttributeDataTypes 
		{
			BOOLEAN = 1,
			DATE = 3,
			DOUBLE = 4,
			INTEGER = 5,
			STRING = 6,
			BINARY = 7,
			CLOB = 9,
			DATETIME = 10,
			VIRTUAL = -1
		}

		/// <summary>
		/// Constant values specifying types of Component Definitions
		/// Copied from the IBM DB2 Records Manager API
		/// com.ibm.gre.engine.util.ComponentDefinitionTypes class  
		/// </summary>
		public enum ComponentDefinitionTypes
		{
			ATTRIBUTE_DEFINITION = -2,
			AUDIT_CONFIGURATION = -39,
			AUDIT_ENTRY = -17,
			AUTO_CLASSIFY_RULE = -21,
			CHARGE_OUT = -20,
			CHARGE_OUT_HISTORY = -25,
			DEFAULT = -6,
			DERIVATIVE_SOURCE = -30,
			DISPOSAL_AUTHORITY = -15,
			FILE_PLAN_COMPONENT = 0,
			FILE_PLAN_COMPONENT_DEFINITION = -1,
			FILE_PLAN_COMPONENT_NODE = -38,
			FILE_PLAN_VIEW = -13,
			GROUP = -4,
			HOST = -12,
			LIFE_CYCLE_CODE = -10,
			LIFE_CYCLE_CODE_PHASE = -18,
			LIFE_CYCLE_PHASE = -9,
			LOGIC_EXTENSION = -8,
			PROFILE = -16,
			RELATIONSHIP_DEFINITION = -5,
			REPORT_LAYOUT = -23,
			RESERVATION = -19,
			SAVED_QUERY = -24,
			SECURITY_CLASSIFICATION = -28,
			SECURITY_CLASSIFICATION_EVENT = -33,
			SECURITY_CLASSIFICATION_EXEMPTION = -32,
			SECURITY_CLASSIFICATION_GUIDE = -36,
			SECURITY_CLASSIFICATION_GUIDE_ENTRY = -37,
			SECURITY_CLASSIFICATION_INFO = -29,
			SECURITY_CLASSIFICATION_REASON = -31,
			SECURITY_DESCRIPTOR = -34,
			SUPER_USER_GROUP = -35,
			SUSPENSION = -14,
			USER = -3,
			VITAL_RECORD_IDENTIFIER = -27
		}

		/// <summary>
		/// Constant values specifying types of UI controls
		/// Copied from the IBM DB2 Records Manager API
		/// com.ibm.gre.engine.util.ProfileControlType class  
		/// </summary>
		public enum ProfileControlType
		{
			SINGLELINE = 1,
			MULTILINE = 2,
			LINESEPERATOR = 3,
			BLANKLINE = 4,
			GROUPHEADER = 5,
			GROUPEND = 6,
			USERFIELD = 7,
			COMBOBOX = 8,
			DROPLIST = 9, 
			MULTISELECT_LIST = 10,
			MULTI_VALUE = 11
		}

		/// <summary>
		/// See com.ibm.gre.engine.types.ConflictTypes
		/// </summary>
		public enum ConflictTypes
		{
			ACTION_CONFLICT = 1, 
			ELIGIBLE_DATE_CONFLICT = 2, 
			MORE_THEN_ONE_CONFLICT = 3 
		}

		//Utility Constants

		/// <summary>
		/// Non Records Manager constants representing
		/// the type of line that should be rendered
		/// on a Query Form.
		/// </summary>
		public enum LineType
		{
			NONE,
			SCOPE,
			GT,
			LT,
			EQ,
			GTEQ,
			LTEQ,
			LIKE,
			NOT_LIKE,
			IS_NULL,
			LB,
			AND,
			OR,
			NOT,
			RB,
			HAS_CHILD,
			IS_CHILD_OF,
			HAS_PART,
			IS_PART_OF
		}

	}
}
