<?xml version="1.0" encoding="UTF-8"?>

<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" 
				version="1.0" 
				xmlns:fo="http://www.w3.org/1999/XSL/Format"
				xmlns:exsl="http://exslt.org/common"
				extension-element-prefixes="exsl">
	<xsl:variable name="numberOfColumn" select="exsl:node-set($config)/List/Item[@name='NumberOfColumn']"/>	
	<xsl:variable name="sysFullPathPrefix" select="exsl:node-set($config)/List/Item[@name='sysFullPathPrefix']"/>	
	<xsl:variable name="sysFullPathLevelDelimiter" select="exsl:node-set($config)/List/Item[@name='sysFullPathLevelDelimiter']"/>	
	<xsl:variable name="sysPartitionLevelDelimiter" select="exsl:node-set($config)/List/Item[@name='sysPartitionLevelDelimiter']"/>	

	<xsl:template match="/">
		<fo:root xmlns:fo="http://www.w3.org/1999/XSL/Format">

			<!-- defines page layout -->
			<fo:layout-master-set>
				<fo:simple-page-master master-name="simple" page-width="28cm" page-height="22cm" margin-top="0.5cm" margin-bottom="0.5cm" margin-left="0.5cm" margin-right="0.5cm">
					<fo:region-before extent="0.5cm"/>
					<fo:region-body margin-top="0.5cm" margin-bottom="0.5cm"/>
				</fo:simple-page-master>
			</fo:layout-master-set>


			<!-- defines the content -->
			<fo:page-sequence master-reference="simple">
				<fo:static-content flow-name="xsl-region-before" font-size="8pt">
					<fo:block  text-align="right">
					<fo:page-number/>
					<fo:inline>/</fo:inline>
					<fo:page-number-citation ref-id="terminator"/>
					</fo:block>
				</fo:static-content>

				<fo:flow flow-name='xsl-region-body'>
					<fo:block font-size="14pt" font-weight="bold" text-align="center">
					File Plan Report
					</fo:block>
				
					<fo:table font-size="8pt" border="1">


						<xsl:variable name="width" select="27 div ($numberOfColumn + 1)"/>
						
						<xsl:call-template name="generateColumnWidth">
							<xsl:with-param name="numberOfColumn" select="$numberOfColumn + 1"/>
							<xsl:with-param name="width" select="$width"/>
						</xsl:call-template>
						


						
						<!-- Header -->
						<!--						
						<fo:table-header font-weight="bold" color="white" background-color="gray">
						</fo:table-header>
						-->
						<!-- File Plan -->
						<fo:table-body>
							<xsl:for-each select="/ResultList/Data/List/Item">
								<xsl:variable name="curFlPlnCmpntDefNm" select="@TS_FlPlnCmpntDef.FlPlnCmpntDefNm"/>
								<xsl:variable name="isReport" 
									select="exsl:node-set($config)/List/Item[@name = 'definition'][@irmSystem = $curFlPlnCmpntDefNm]/@isReport"/>
								
								<xsl:if test="$isReport = 'yes' ">

								<fo:table-row>					
									<xsl:variable name="fullPath" select="@TS_FlPlnNode.AddressByName"/>
									<xsl:variable name="fullPath1" select="substring-after($fullPath, $sysFullPathPrefix)"/>											
									<xsl:variable name="view1" select="substring-before($fullPath1, $sysFullPathLevelDelimiter)"/>
									<xsl:call-template name="writeItem">	
										<xsl:with-param name="curPrefix" select="concat($sysFullPathPrefix,$view1,$sysFullPathLevelDelimiter)"/>
										<xsl:with-param name="curColumn" select="$numberOfColumn"/>
										<xsl:with-param name="fullPath" select="$fullPath"/>
									</xsl:call-template>

									<fo:table-cell padding="2pt" border="solid black">
										<fo:block text-align="start">
										<xsl:value-of select="$curFlPlnCmpntDefNm" />
										</fo:block>
									</fo:table-cell>

								</fo:table-row>									
								</xsl:if>
								
							</xsl:for-each>
						</fo:table-body>
					</fo:table>

					<fo:block id="terminator"/>

				</fo:flow>

			</fo:page-sequence>
		</fo:root>
	</xsl:template>

	<xsl:template name="generateColumnWidth">
		<xsl:param name="numberOfColumn"/>
		<xsl:param name="width"/>

		<fo:table-column >
			<xsl:attribute name="column-width">
				<xsl:value-of select="$width"/>
				<xsl:text>cm</xsl:text> 
			</xsl:attribute>
		</fo:table-column>

		<xsl:if test="$numberOfColumn > 1">
		   <xsl:call-template name="generateColumnWidth">
		    <xsl:with-param name="numberOfColumn" select="$numberOfColumn - 1"/>
		    <xsl:with-param name="width" select="$width"/>
		   </xsl:call-template>
		 </xsl:if>
		
	</xsl:template>



	<xsl:template name="writeItem">
		<xsl:param name="curPrefix"/>	
		<xsl:param name="curColumn"/>					
		<xsl:param name="fullPath"/>
		
		<xsl:variable name="curSuffix" select="substring-after($fullPath,$curPrefix)"/>
		<xsl:variable name="newPrefix" select="substring-before($curSuffix, $sysFullPathLevelDelimiter)"/>
		<xsl:if test="string-length($newPrefix) = 0">
			<fo:table-cell padding="2pt" border="solid black">
				<fo:block text-align="start">
				<xsl:variable name="partName" select="substring-after($curSuffix, $sysPartitionLevelDelimiter)"/>
				<xsl:if test="string-length($partName) = 0">
					<xsl:value-of select="$curSuffix" />
				</xsl:if>

				<xsl:if test="string-length($partName) > 0">
					<xsl:value-of select="$partName" />
				</xsl:if>

				</fo:block>
			</fo:table-cell>
			
			<xsl:call-template name="writeItemEmptyColumn">
				<xsl:with-param name="emptyColumn" select="$curColumn - 1"/>
			</xsl:call-template>
		</xsl:if>

		<xsl:if test="string-length($newPrefix) > 0">
			<fo:table-cell padding="2pt" border="solid black">
			</fo:table-cell>
			
			<xsl:call-template name="writeItem">
				<xsl:with-param name="curPrefix" select="concat($curPrefix,$newPrefix,$sysFullPathLevelDelimiter)"/>
				<xsl:with-param name="curColumn" select="$curColumn - 1"/>
				<xsl:with-param name="fullPath" select="$fullPath"/>
			</xsl:call-template>

		</xsl:if>
		
	</xsl:template>

	<xsl:template name="writeItemEmptyColumn">
		<xsl:param name="emptyColumn"/>
		
		<xsl:if test="$emptyColumn > 0">
			<fo:table-cell padding="2pt" border="solid black">
			</fo:table-cell>
			<xsl:call-template name="writeItemEmptyColumn">
				<xsl:with-param name="emptyColumn" select="$emptyColumn - 1"/>			
			</xsl:call-template>
		</xsl:if>
	</xsl:template>
	
	<!-- ======================================================================================================= 
		This report layout is to generate a PRO file plan report. 
		
		Report On : File Plan Node
		Out Fields: File Plan Node - Address 
					File Plan Component Definition - Component Definition Name
		Sort on	  : File Plan Node - Address 
		
		Configuration 
		1)The default configuration only show Class, Folder and Part. It doesn't show component of record type.
		
		To add the type of component to the report, add this entry to the configuration list, for example:
		
		<Item name="definition" irmSystem ="E-mail" isReport="yes"/>		
		
		2) You can also change the System Full Path Prefix, System Full Path Level Delimiter and System Partition 
			Level Delimiter.
		
		See below
	
		========================================================================================================
	-->
	
	
	<xsl:variable name="config">
		<List>
			<!-- NumberOfColumn is the number of column for file plan components. It doesn't include the column for file plan definition
				 NumberOfColumn = <Maximum layers of class> + 1 (folder) + 1(record)
			-->
			<Item name="NumberOfColumn">6</Item>
			<Item name="sysFullPathPrefix">\\</Item>
			<Item name="sysFullPathLevelDelimiter">\</Item>
			<Item name="sysPartitionLevelDelimiter">.</Item>
			<Item name="definition" irmSystem ="Class" isReport="yes"/>
			<Item name="definition" irmSystem ="Folder" isReport="yes"/>
			<Item name="definition" irmSystem ="Part" isReport="yes"/>
			<Item name="definition" irmSystem ="Default Record" isReport="no"/>
		</List>

	</xsl:variable>

</xsl:stylesheet>