/*
 * IBM Confidential
 * OCO Source Materials
 * 5724-E68
 * 
 *  Copyright IBM Corp. 2003
 * The source code for this program is not published or otherwise
 * divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 * 
 */
package com.ibm.gre.pro.extension;

import java.io.IOException;
import java.io.InputStream;
import java.util.EventObject;
import java.util.HashMap;
import java.util.Properties;

import javax.ejb.CreateException;
import javax.naming.NamingException;
import javax.rmi.PortableRemoteObject;

import org.apache.log4j.Logger;
import org.w3c.dom.Document;
import org.w3c.dom.Node;

import com.ibm.gre.engine.ejb.business.util.EJBLocator;
import com.ibm.gre.engine.ejb.facade.FilePlanComponentControllerEJBLocal;
import com.ibm.gre.engine.ejb.facade.FilePlanComponentControllerEJBLocalHome;
import com.ibm.gre.engine.extension.FilePlanComponentListener;
import com.ibm.gre.engine.util.AuditComponentActions;
import com.ibm.gre.engine.util.BusinessComponentsJNDINames;
import com.ibm.gre.engine.util.BusinessLayerError;
import com.ibm.gre.engine.util.ExtensionParamKeys;
import com.ibm.gre.engine.util.RecordManagerException;
import com.ibm.gre.engine.util.RecordManagerExceptionTypes;
import com.ibm.gre.engine.util.ServiceLocator;
import com.ibm.gre.engine.util.XMLUtil;
import com.ibm.gre.engine.value.DBConfig;
import com.ibm.gre.engine.value.StateMachine;
import com.ibm.gre.engine.value.StateMachineKeys;
import com.ibm.gre.engine.value.User;

/** 
 * @version 	1.0
 * @author		Tom Chen
 * 
 * Purpose: add audit entry on the update of field Last Review Date
 * Handles the following events:
 *	beforeUpdateEvent
 * 
 * IRM Configuration Requirements
 * ------------------------------
 * 
 * 
 * 
 * Building this project
 * ---------------------
 * 
 *  Tips for running within IDE with Engine projects:
 *  ----------------
 * 
 * ---------------- 
 * Revision History
 * ----------------
 * Date				Author		Changes
 * -------------------------------------
 * Aug 20, 2004		Tom Chen	Created
 * 
 */
public class PROFilePlanComponentListener implements FilePlanComponentListener {

	// Application Resource Constants
	final private static String APP_RES_FILE_NAME = "ApplicationResources.properties";
	private static Logger logger = Logger.getLogger("EXTENSION");
	//	ApplicationResources.properties
	private static Properties prop; 	
	/**
	 * @see com.ibm.gre.engine.extension.FilePlanComponentListener#beforeAddEvent(EventObject)
	 */
	public void beforeAddEvent(EventObject e) throws RecordManagerException {
	}

	/**
	 * @see com.ibm.gre.engine.extension.FilePlanComponentListener#afterAddEvent(EventObject)
	 */
	public void afterAddEvent(EventObject e) throws RecordManagerException {
	}

	/**
	 * @see com.ibm.gre.engine.extension.FilePlanComponentListener#beforeDeleteEvent(EventObject)
	 */
	public void beforeDeleteEvent(EventObject e) throws RecordManagerException {
	}

	/**
	 * @see com.ibm.gre.engine.extension.FilePlanComponentListener#afterDeleteEvent(EventObject)
	 */
	public void afterDeleteEvent(EventObject e) throws RecordManagerException {
	}

	/**
	 * @see com.ibm.gre.engine.extension.FilePlanComponentListener#beforeGetByBarCodeEvent(EventObject)
	 */
	public void beforeGetByBarCodeEvent(EventObject e) throws RecordManagerException {
	}

	/**
	 * @see com.ibm.gre.engine.extension.FilePlanComponentListener#afterGetByBarCodeEvent(EventObject)
	 */
	public void afterGetByBarCodeEvent(EventObject e) throws RecordManagerException {
	}

	/**
	 * @see com.ibm.gre.engine.extension.FilePlanComponentListener#beforeUpdateEvent(EventObject)
	 */
	public void beforeUpdateEvent(EventObject e) throws RecordManagerException {
		logger.info("beforeUpdateEvent - BEGIN");

		HashMap map = (HashMap)e.getSource();

		String fpcXML = (String)map.get(ExtensionParamKeys.FILE_PLAN_COMPONENT_XML);		
		Document doc = XMLUtil.getDocument(fpcXML);
		
		Node newLastReviewDateNode = XMLUtil.getNode(doc, "Field", "FlPlnCmpntAttrDefColNm" , "LastRvwDt");
		
		if (newLastReviewDateNode == null)
			return;
			
		String newLastReviewDate = null;
		
		
		newLastReviewDate = XMLUtil.getStringValue(newLastReviewDateNode);

		long fpcId = ((Long)map.get(ExtensionParamKeys.FILE_PLAN_COMPONENT_ID)).longValue();
		StateMachine stateMachine = (StateMachine)map.get(ExtensionParamKeys.STATE_MACHINE);
		//get old last review date
		String oldLastReviewDate = null;
		FilePlanComponentControllerEJBLocal fpccl = null;
		try{	
			ServiceLocator locator = ServiceLocator.getInstance();
			Object obj = locator.lookup("local:ejb/" + BusinessComponentsJNDINames.FILE_PLAN_COMPONENT_CONTROLLER_JNDI);
			
			FilePlanComponentControllerEJBLocalHome fpcch = (FilePlanComponentControllerEJBLocalHome) PortableRemoteObject.narrow(obj, FilePlanComponentControllerEJBLocalHome.class);
			fpccl = fpcch.create(stateMachine);			
		}
		catch(NamingException ex){
			RecordManagerException rme = new RecordManagerException(BusinessLayerError.JNDI_LOOKUP_FAILED, ex.getMessage(), getClass().getName(), RecordManagerExceptionTypes.BUSINESS_LOGIC, ex);
			rme.addMessageParameter(ex.getMessage());
			throw rme;
		}
		catch(CreateException ce){
			logger.debug("Extension exception:", ce);			
			throw new RecordManagerException(BusinessLayerError.UNKNOWN, ce.getMessage(), getClass().getName(), RecordManagerExceptionTypes.SYSTEM, ce);
		}

		String oldXML = fpccl.getFilePlanComponentById(fpcId);
		Document oldDoc = XMLUtil.getDocument(oldXML);
		oldLastReviewDate = XMLUtil.getOptionalFieldStringValue(oldDoc, "FlPlnCmpntAttrDefColNm" ,"LastRvwDt"); 
		
		
		//last review date is NOT changed
		if (newLastReviewDate == null) newLastReviewDate="";
		if (oldLastReviewDate == null) oldLastReviewDate="";
		if 	(newLastReviewDate.equals(oldLastReviewDate) ) {
			logger.info("last review date is not changed. No audit entry is logged.(FPC ID:" + fpcId + ")");
			return; 
		}
			
		
		String fieldName = getProperty("custom.field.review_decision");
		String reviewDecision = null;
			
		Node node = XMLUtil.getNode(doc, "Field", "FlPlnCmpntAttrDefColNm", fieldName);
		if (node != null) {
			reviewDecision = XMLUtil.getStringValue(node);	
		} else {
			reviewDecision = XMLUtil.getOptionalFieldStringValue(oldDoc, "FlPlnCmpntAttrDefColNm" , fieldName); 
			
		}
		
		//add audit entry start
		Node firstNode = XMLUtil.getNodeByXPath(oldDoc, "/Fields/Field[@FlPlnCmpntAttrDefColNm='FlPlnCmpntID']/@FlPlnCmpntDefID");
		long definitionId = XMLUtil.getLongValue(firstNode);
		DBConfig dbConfig = (DBConfig)stateMachine.getAttribute(StateMachineKeys.DATASOURCE);
		User user = (User)stateMachine.getAttribute(StateMachineKeys.USER); 
		String subEventName = getProperty("sub.event.name");
		EJBLocator.getAuditEntryEJB().addEntryWithSubEventName(dbConfig, user, definitionId, AuditComponentActions.UPDATE, subEventName, fpcId, reviewDecision, null);
		//add audit entry end
		logger.info("audit entry will be logged when the updating is committed.(Review Decision:" + reviewDecision + ")");
		
		logger.info("beforeUpdateEvent - END");				

	}

	/**
	 * @see com.ibm.gre.engine.extension.FilePlanComponentListener#afterUpdateEvent(EventObject)
	 */
	public void afterUpdateEvent(EventObject e) throws RecordManagerException {
	}

	/**
	 * @see com.ibm.gre.engine.extension.FilePlanComponentListener#beforeGetByIDEvent(EventObject)
	 */
	public void beforeGetByIDEvent(EventObject e) throws RecordManagerException {
	}

	/**
	 * @see com.ibm.gre.engine.extension.FilePlanComponentListener#afterGetByIDEvent(EventObject)
	 */
	public void afterGetByIDEvent(EventObject e) throws RecordManagerException {
	}

	/**
	 * @see com.ibm.gre.engine.extension.FilePlanComponentListener#beforeSetFieldEvent(EventObject)
	 */
	public void beforeSetFieldEvent(EventObject e) throws RecordManagerException {
	}

	/**
	 * @see com.ibm.gre.engine.extension.FilePlanComponentListener#afterSetFieldEvent(EventObject)
	 */
	public void afterSetFieldEvent(EventObject e) throws RecordManagerException {
	}

	/**
	 * @see com.ibm.gre.engine.extension.FilePlanComponentListener#beforeGetFieldEvent(EventObject)
	 */
	public void beforeGetFieldEvent(EventObject e) throws RecordManagerException {
	}

	/**
	 * @see com.ibm.gre.engine.extension.FilePlanComponentListener#afterGetFieldEvent(EventObject)
	 */
	public void afterGetFieldEvent(EventObject e)
		throws RecordManagerException {
	}
	/**
	 * @see com.ibm.gre.engine.extension.FilePlanComponentListener#afterTransitionEvent(EventObject)
	 */
	public void afterTransitionEvent(EventObject e) throws RecordManagerException {
	}

	/**
	 * @see com.ibm.gre.engine.extension.FilePlanComponentListener#beforeTransitionEvent(EventObject)
	 */
	public void beforeTransitionEvent(EventObject e) throws RecordManagerException {
	}
	
	/**
	 * @see com.ibm.gre.engine.extension.FilePlanComponentListener#beforeSupercedeEvent(java.util.EventObject)
	 */
	public void beforeSupercedeEvent(EventObject e) throws RecordManagerException {
	}

	/**
	 * @see com.ibm.gre.engine.extension.FilePlanComponentListener#afterSupercedeEvent(java.util.EventObject)
	 */
	public void afterSupercedeEvent(EventObject e) throws RecordManagerException {
	}



	static public String getProperty(String key) throws RecordManagerException{
		
		if (prop == null) {
			InputStream is = PROFilePlanComponentListener.class.getResourceAsStream(APP_RES_FILE_NAME);
			prop = new Properties();
			try{
				prop.load(is);
			}
			catch(IOException e){
				logger.info("initApplicationResources - Properties.load failed ",e);
				throw new RecordManagerException(BusinessLayerError.UNKNOWN, e.getMessage(), PROFilePlanComponentListener.class.getName(), RecordManagerExceptionTypes.SYSTEM, e);
			}
			
		}

		return prop.getProperty(key); 
	}

}
