/*
 * Created on 17-Jan-05
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package com.ibm.gre.pro.extension;
import java.io.IOException;
import java.io.InputStream;
import java.rmi.RemoteException;
import java.text.MessageFormat;
import java.util.EventObject;
import java.util.HashMap;
import java.util.Properties;

import javax.naming.InitialContext;
import javax.rmi.PortableRemoteObject;

import org.apache.log4j.Logger;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import com.ibm.gre.engine.ejb.facade.ClientControllerEJB;
import com.ibm.gre.engine.ejb.facade.DefinitionControllerEJB;
import com.ibm.gre.engine.ejb.facade.LoginManagerEJB;
import com.ibm.gre.engine.ejb.facade.LoginManagerEJBHome;
import com.ibm.gre.engine.ejb.facade.TemplateControllerEJB;
import com.ibm.gre.engine.extension.FilePlanComponentListener;
import com.ibm.gre.engine.util.BusinessComponentsJNDINames;
import com.ibm.gre.engine.util.DAOLayerErrors;
import com.ibm.gre.engine.util.ExtensionParamKeys;
import com.ibm.gre.engine.util.RecordManagerException;
import com.ibm.gre.engine.util.RecordManagerExceptionTypes;
import com.ibm.gre.engine.util.XMLUtil;
import com.ibm.gre.engine.value.DBConfig;
import com.ibm.gre.engine.value.StateMachine;
import com.ibm.gre.engine.value.StateMachineKeys;

/** 
 * @version 	1.0
 * @author		brant
 * 
 * Purpose: the purpose is to handle PRO requirement to have only one security descriptor
 * Handles the following events:
 *	beforeAddEvent
 *	beforeUpdateEvent
 * 
 * Building this project
 * ---------------------
 * 1) Make sure you have the lastest IRMAPI.jar and log4j-1.2.7.jar in your project root directory.
 * Change your Java Build Path Libraries to reference them.
 * 
 * 2) Export project as a jar, call it IRMDocumentExtension.jar include
 * package com.ibm.gre.dod.extension, do not include IRMAPI.jar and log4j-1.2.7.jar these
 * will be picked up from the engine class path when the class is loaded.
 * 
 * 
 *  Tips for running within IDE with Engine projects:
 *  ----------------
 * 	1) Modify Trafalgar (EAR) 
 * 		Deployment Descriptor - Module
 * 		Add (Project Utility JARs) to include IRMDocumentExtension.jar
 * 		Save it
 *  2) Modify MocharEJB MANIFEST.MF to include IRMDocumentExtension.jar as Dependencies
 *  3) To build IRMDocumentExtension, add project references to MochaEJB and MochaJava
 * 
 * ---------------- 
 * Revision History
 * ----------------
 * Date				Author		Changes
 * -------------------------------------
 * Jan 17, 2005		Brant		Created
 * 
 */
public class PROSecurityDescriptorListener implements FilePlanComponentListener {

	private String COMP_NAME_XPATH = "//List/Item";
	private String adminPassword;
	private String adminUserName;
	private int definitionID;
	private InitialContext context;
	private String errorMessage;
	private static final String APP_RES_ERROR_MSG = "error.message";
	private Properties prop;
	private static final String APP_RES_FILE_NAME = "PROResources.properties";
	final private static String APP_RES_LOGIN_NAME = "login.user.name";
	final private static String APP_RES_LOGIN_PW = "login.user.pw";
	private static String ADMIN_USER_NAME = "administrator";
	private static String ADMIN_PASSWORD = "cronos";
	//	component definition ID of the fpc
	private long fpcCmpntDefID ;
	// client controller that logs in as administrator - for changePermission operation
	private ClientControllerEJB adminClientController;
//	error constants	
	 private static int E_EXTSN_BASE = 800000;
	 private static int E_EXTSN_LOGIN_FAILED = E_EXTSN_BASE + 1; // failed login - required Administrator/cronos
	 private static int E_EXTSN_NO_CLNT_CNTRLR = E_EXTSN_BASE + 2; // failed to create adminClientController
	 private static int E_EXTSN_NO_USR_CNTRLR = E_EXTSN_BASE + 3; // failed to create user controller
	 private static int E_EXSTN_BAD_XML = E_EXTSN_BASE + 4; // parsing xml failed.
	 private static int E_EXSTN_NO_ACCESS_TO_FPC = E_EXTSN_BASE + 5;// no access to Fpc
	 private static int E_EXSTN_NO_CST_PROJECT = E_EXTSN_BASE + 6;// missing CST_PROJECT in user
	 //private static int E_EXSTN_NO_CST_SUPPMARK = E_EXTSN_BASE + 7;// missing CST_SUPPMARK in user
	 private static int E_EXSTN_ALLOW_ACCESS = E_EXTSN_BASE + 8; // allowAccess failed - unknown exception
	 private static int E_EXSTN_SET_PERMISSIONS = E_EXTSN_BASE + 9; // setPermissions failed
	 private static int E_EXTSN_NO_QRY_CNTRLR = E_EXTSN_BASE + 10; // failed to create query controller
//	Application Resouce Constants for error string
	 private static String E_RES_LOGIN_BAD_USR_PW = "error.login.bad_user_pw";
	 private static String E_RES_NO_ACCESS_ADD = "error.noaccess.add";
	 private static String E_RES_NO_ACCESS_FETCH = "error.noaccess.fetch";
	 private static String E_RES_NO_USR_CST_FIELD = "error.user.missing_custom_attribute";
	 
	 
	/* (non-Javadoc)
	 * @see com.ibm.gre.engine.extension.FilePlanComponentListener#beforeAddEvent(java.util.EventObject)
	 */
	private static String SEC_DESC_XPATH = "//Fields/Field[@FlPlnCmpntAttrDefColNm=\"SecDescLst\"]/List/Item";
	private static Logger logger = Logger.getLogger("PROEXTENSION");
	public void beforeAddEvent(EventObject e) throws RecordManagerException {
		//before the add, check to see if there is more than one security descriptor
		//if there is more than one security descriptor, throw an error
		logger.info("beforeAddEvent - BEGIN");
		HashMap map = (HashMap) e.getSource();
		StateMachine stateMachine = (StateMachine) map.get(ExtensionParamKeys.STATE_MACHINE);
		
		//get the xml for the newly added component
		String fpcXml = (String) map.get(ExtensionParamKeys.FILE_PLAN_COMPONENT_XML);
		//check the xml for more than one descriptor
		int numberOfDescriptors = countNumberOfSecurityDescriptor(fpcXml);
		if (numberOfDescriptors>1){
			throwTooManyDescriptorsException(e);
		}
	}

	/**
	 * 
	 */
	private void throwTooManyDescriptorsException(EventObject e) throws RecordManagerException {
		
		initApplicationResources();
		String compName = "";
		String message = "";
		try {
			compName = getComponentName(e);
		} catch (RemoteException re){
			compName = "#" + String.valueOf(getDefinitionID(e));
		}
		
		message = getMessage(APP_RES_ERROR_MSG, compName);
		throw new RecordManagerException(99999, message, "PROListener", 1);
		
	}

	/**
	 * @param e
	 * @return
	 */
	private String getComponentName(EventObject e) throws RemoteException, RecordManagerException {
		HashMap map = (HashMap) e.getSource();
		StateMachine stateMachine = (StateMachine) map.get(ExtensionParamKeys.STATE_MACHINE);
		loginAsAdministrator(stateMachine);
		
		TemplateControllerEJB tc = getTemplateController();
		DefinitionControllerEJB dc = getDefinitionController();
		
		long defID = getDefinitionID(e);
		String xml = dc.getComponentDefinitionById(defID);
		
		//String compDefNm = tc.getSystemComponentField(ComponentDefinitionTypes.FILE_PLAN_COMPONENT_DEFINITION, defID, "FlPlnCmpntDefNm");
		String compDefNm = extractCompDefNameFromXML(xml);
		
		return compDefNm;
		
	}

	/**
	 * @param xml
	 * @return
	 */
	private String extractCompDefNameFromXML(String xml) throws RecordManagerException {
		Document xmlDoc = XMLUtil.getDocument(xml);
		Node compNameXPath = XMLUtil.getNodeByXPath(xmlDoc, COMP_NAME_XPATH);
		String compName = XMLUtil.getStringValueFromXMLAttribute(compNameXPath, "FlPlnCmpntDefNm");
		return compName;
	}

	/**
	 * @return
	 */
	private long getDefinitionID(EventObject e) {
		HashMap map = (HashMap) e.getSource();
		Long defID = (Long)map.get("definitionId");
		try {
			return defID.longValue();
		} catch (Exception ex){
			return 0;
		}
	}

	/**
	 * @param fpcXml
	 * @return
	 */
	private int countNumberOfSecurityDescriptor(String fpcXml) throws RecordManagerException {
		//check the xml and count the number of security descriptors entered
		int descriptors = 0;
		Document fpcDoc = XMLUtil.getDocument(fpcXml);
		NodeList securityDescriptors = XMLUtil.getNodeListByXPath(fpcDoc, SEC_DESC_XPATH);
		
		if (securityDescriptors!=null){
			descriptors = securityDescriptors.getLength();
		};
		return descriptors;
	}

	/* (non-Javadoc)
	 * @see com.ibm.gre.engine.extension.FilePlanComponentListener#afterAddEvent(java.util.EventObject)
	 */
	public void afterAddEvent(EventObject e) throws RecordManagerException {
		// TODO Auto-generated method stub

	}

	/* (non-Javadoc)
	 * @see com.ibm.gre.engine.extension.FilePlanComponentListener#beforeDeleteEvent(java.util.EventObject)
	 */
	public void beforeDeleteEvent(EventObject e)
		throws RecordManagerException {
		// TODO Auto-generated method stub

	}

	/* (non-Javadoc)
	 * @see com.ibm.gre.engine.extension.FilePlanComponentListener#afterDeleteEvent(java.util.EventObject)
	 */
	public void afterDeleteEvent(EventObject e) throws RecordManagerException {
		// TODO Auto-generated method stub

	}

	/* (non-Javadoc)
	 * @see com.ibm.gre.engine.extension.FilePlanComponentListener#beforeGetByBarCodeEvent(java.util.EventObject)
	 */
	public void beforeGetByBarCodeEvent(EventObject e)
		throws RecordManagerException {
		// TODO Auto-generated method stub

	}

	/* (non-Javadoc)
	 * @see com.ibm.gre.engine.extension.FilePlanComponentListener#afterGetByBarCodeEvent(java.util.EventObject)
	 */
	public void afterGetByBarCodeEvent(EventObject e)
		throws RecordManagerException {
		// TODO Auto-generated method stub

	}

	/* (non-Javadoc)
	 * @see com.ibm.gre.engine.extension.FilePlanComponentListener#beforeUpdateEvent(java.util.EventObject)
	 */
	public void beforeUpdateEvent(EventObject e)
		throws RecordManagerException {
		//before the add, check to see if there is more than one security descriptor
		//if there is more than one security descriptor, throw an error
		logger.info("beforeUpdateEvent - BEGIN");
		HashMap map = (HashMap) e.getSource();
		StateMachine stateMachine = (StateMachine) map.get(ExtensionParamKeys.STATE_MACHINE);
	
		//get the xml for the newly added component
		String fpcXml = (String) map.get(ExtensionParamKeys.FILE_PLAN_COMPONENT_XML);
		//check the xml for more than one descriptor
		int numberOfDescriptors = countNumberOfSecurityDescriptor(fpcXml);
		if (numberOfDescriptors>1){
			throwTooManyDescriptorsException(e);
		}

	}

	/* (non-Javadoc)
	 * @see com.ibm.gre.engine.extension.FilePlanComponentListener#afterUpdateEvent(java.util.EventObject)
	 */
	public void afterUpdateEvent(EventObject e) throws RecordManagerException {
		// TODO Auto-generated method stub

	}

	/* (non-Javadoc)
	 * @see com.ibm.gre.engine.extension.FilePlanComponentListener#beforeGetByIDEvent(java.util.EventObject)
	 */
	public void beforeGetByIDEvent(EventObject e)
		throws RecordManagerException {
		// TODO Auto-generated method stub

	}

	/* (non-Javadoc)
	 * @see com.ibm.gre.engine.extension.FilePlanComponentListener#afterGetByIDEvent(java.util.EventObject)
	 */
	public void afterGetByIDEvent(EventObject e)
		throws RecordManagerException {
		// TODO Auto-generated method stub

	}

	/* (non-Javadoc)
	 * @see com.ibm.gre.engine.extension.FilePlanComponentListener#beforeSetFieldEvent(java.util.EventObject)
	 */
	public void beforeSetFieldEvent(EventObject e)
		throws RecordManagerException {
		// TODO Auto-generated method stub

	}

	/* (non-Javadoc)
	 * @see com.ibm.gre.engine.extension.FilePlanComponentListener#afterSetFieldEvent(java.util.EventObject)
	 */
	public void afterSetFieldEvent(EventObject e)
		throws RecordManagerException {
		// TODO Auto-generated method stub

	}

	/* (non-Javadoc)
	 * @see com.ibm.gre.engine.extension.FilePlanComponentListener#beforeGetFieldEvent(java.util.EventObject)
	 */
	public void beforeGetFieldEvent(EventObject e)
		throws RecordManagerException {
		// TODO Auto-generated method stub

	}

	/* (non-Javadoc)
	 * @see com.ibm.gre.engine.extension.FilePlanComponentListener#afterGetFieldEvent(java.util.EventObject)
	 */
	public void afterGetFieldEvent(EventObject e)
		throws RecordManagerException {
		// TODO Auto-generated method stub

	}

	/* (non-Javadoc)
	 * @see com.ibm.gre.engine.extension.FilePlanComponentListener#beforeTransitionEvent(java.util.EventObject)
	 */
	public void beforeTransitionEvent(EventObject e)
		throws RecordManagerException {
		// TODO Auto-generated method stub

	}

	/* (non-Javadoc)
	 * @see com.ibm.gre.engine.extension.FilePlanComponentListener#afterTransitionEvent(java.util.EventObject)
	 */
	public void afterTransitionEvent(EventObject e)
		throws RecordManagerException {
		// TODO Auto-generated method stub

	}

	/* (non-Javadoc)
	 * @see com.ibm.gre.engine.extension.FilePlanComponentListener#beforeSupercedeEvent(java.util.EventObject)
	 */
	public void beforeSupercedeEvent(EventObject e)
		throws RecordManagerException {
		// TODO Auto-generated method stub

	}

	/* (non-Javadoc)
	 * @see com.ibm.gre.engine.extension.FilePlanComponentListener#afterSupercedeEvent(java.util.EventObject)
	 */
	public void afterSupercedeEvent(EventObject e)
		throws RecordManagerException {
		// TODO Auto-generated method stub

	}
	
	
	
	private void initApplicationResources () {
		logger.info("initApplicationResources - BEGIN");
		InputStream is = PROSecurityDescriptorListener.class.getResourceAsStream(APP_RES_FILE_NAME);
		prop = new Properties();
		if(null != is){
			try{
				prop.load(is);
			}
			catch(IOException e){
				logger.info("initApplicationResources - Properties.load failed ",e);
			}
		}
		else{
			logger.info("initApplicationResources - getResoucesAsStream is null");
		}
		errorMessage = prop.getProperty(APP_RES_ERROR_MSG);
		adminUserName = prop.getProperty(APP_RES_LOGIN_NAME,ADMIN_USER_NAME);
		adminPassword = prop.getProperty(APP_RES_LOGIN_PW,ADMIN_PASSWORD);
		
	
		logger.info("initApplicationResources - END");		
	}
	
	private void loginAsAdministrator(StateMachine stateMachine) throws RecordManagerException{

		try{
			logger.info("loginAsAdministrator - Before getLoginManagerEJBHome ...");
			if(null == context){
				context = new InitialContext ();
			}
			Object obj = context.lookup(BusinessComponentsJNDINames.LOGIN_MANAGER_EJB_JNDI);
			LoginManagerEJBHome loginMgrHome = (LoginManagerEJBHome) PortableRemoteObject.narrow(obj, LoginManagerEJBHome.class);

//			logger.info("loginAsAdministrator - after getLoginManagerEJBHome ...");
						
			LoginManagerEJB loginMgr = loginMgrHome.create();
//			logger.info("loginAsAdministrator - Before getDatasource ...");
			DBConfig dbConfig = (DBConfig)stateMachine.getAttribute(StateMachineKeys.DATASOURCE);
			logger.info("loginAsAdministrator - Before login Datasource [" + dbConfig.jndi_name + "] User [" + adminUserName + "]");
			adminClientController = loginMgr.login(adminUserName,adminPassword,dbConfig.jndi_name,"","en");
			loginMgr.remove();
			logger.info("loginAsAdministrator - DONE! ...");
		}
		catch(Exception e){
			logger.info("loginAsAdministrator - caught exception!",e);
			if(e instanceof RecordManagerException ){
				RecordManagerException rme = (RecordManagerException) e;
				if(DAOLayerErrors.USER_NOT_FOUND == rme.getErrorKey()){
					throw new RecordManagerException(E_EXTSN_LOGIN_FAILED, getMessage(E_RES_LOGIN_BAD_USR_PW), getClass().getName(), RecordManagerExceptionTypes.SYSTEM,rme);					
				}
				throw rme;
			}
			else{
				throw new RecordManagerException(E_EXTSN_NO_CLNT_CNTRLR, e.getMessage(), this.getClass().getName(), RecordManagerExceptionTypes.SYSTEM, e);
			}
		}
	}
	private String getMessage(String key){
		return prop.getProperty(key,key);
	}
	private String getMessage(String key, String arg0){
		return getMessage (key,arg0,null,null);
	}	
	private String getMessage(String key, String arg0, String arg1){
		return getMessage(key,arg0,arg1,null);
	}
	private String getMessage(String key, String arg0, String arg1, String arg2){
		String msg = prop.getProperty(key,key);
		String formatedMsg ;
		try{
			String [] args = {arg0, arg1, arg2};
			formatedMsg = MessageFormat.format(msg,args);
		}
		catch(IllegalArgumentException e){
			formatedMsg = key;
		}
		return formatedMsg;
	}
	
	private DefinitionControllerEJB getDefinitionController() throws RecordManagerException{
		try{
			// need filePlanDesignController with Administrator because we need call function gets the file plan definition name
			DefinitionControllerEJB dc = adminClientController.getDefinitionController();
			return dc;			
		}
		catch(Exception e){
			logger.info ("getUserController - caught Exception ", e);			
			throw new RecordManagerException(E_EXTSN_NO_QRY_CNTRLR, e.getMessage(), getClass().getName(), RecordManagerExceptionTypes.SYSTEM, e);
		}
	}
	private TemplateControllerEJB getTemplateController() throws RecordManagerException{
		try{
			// need filePlanDesignController with Administrator because we need call function gets the file plan definition name
			TemplateControllerEJB tc = adminClientController.getTemplateController();
			return tc;			
		}
		catch(Exception e){
			logger.info ("getUserController - caught Exception ", e);			
			throw new RecordManagerException(E_EXTSN_NO_QRY_CNTRLR, e.getMessage(), getClass().getName(), RecordManagerExceptionTypes.SYSTEM, e);
		}
	}
}
